"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = __importDefault(require("path"));
const stylis_1 = require("stylis");
const normalize_path_1 = __importDefault(require("normalize-path"));
const babel_plugin_tw_1 = require("@twstyled/babel-plugin-tw");
const posixSep = path_1.default.posix.sep;
function getProcessorCssText(options) {
    const transformerTwCss = babel_plugin_tw_1.getTransformerTwCss(options);
    return function processCssText({ state, selector, cssText: rawCss, isReferenced }) {
        const { outputFilename, resourceFileName } = getOutputNames(state);
        const twDeclarations = [];
        const cssText = stylis_1.serialize(stylis_1.compile(`${selector}{${rawCss}}`), stylis_1.middleware([
            (element) => {
                if (element.type === stylis_1.DECLARATION && outputFilename) {
                    // When writing to a file, we need to adjust the relative paths inside url(..) expressions
                    // It'll allow css-loader to resolve an imported asset properly
                    element.return = element.value.replace(/\b(url\((["']?))(\.[^)]+?)(\2\))/g, (match, p1, p2, p3, p4) => p1 + transformUrl(p3, outputFilename, resourceFileName) + p4);
                }
            },
            (element) => {
                if (element.type === stylis_1.RULESET &&
                    element.value === 'tailwind' &&
                    element.children) {
                    let result;
                    if (typeof element.children === 'string') {
                        result = element.children;
                    }
                    else {
                        result = element.children
                            .map((c) => c.value.replace(/;$/, ''))
                            .join(' ');
                    }
                    element.value = '';
                    element.children = [];
                    twDeclarations.push(result);
                }
            },
            stylis_1.stringify
        ]));
        if (cssText) {
            state.outputFilename = outputFilename;
            state.resourceFileName = resourceFileName;
            state.file.metadata.cssFilename = path_1.default.relative(path_1.default.dirname(state.file.opts.filename), outputFilename);
        }
        if (twDeclarations.length > 0 && isReferenced) {
            const twclasses = transformerTwCss(twDeclarations, state);
            return {
                cssText,
                twclasses
            };
        }
        return {
            cssText
        };
    };
    //
    // H E L P E R   F U N C T I O N S
    //
    function getOutputNames(state) {
        const { cacheDirectory = './.linaria-cache', extension = '.linaria.module.css' } = state.options || {};
        const root = process.cwd();
        const resourcePath = state.file.opts.filename;
        const baseOutputFileName = resourcePath.replace(/\.[^.]+$/, extension);
        const outputFilename = normalize_path_1.default(path_1.default.join(path_1.default.isAbsolute(cacheDirectory)
            ? cacheDirectory
            : path_1.default.join(process.cwd(), cacheDirectory), resourcePath.includes(root)
            ? path_1.default.relative(root, baseOutputFileName)
            : baseOutputFileName));
        return {
            outputFilename,
            resourceFileName: path_1.default.relative(process.cwd(), resourcePath)
        };
    }
    function transformUrl(url, outputFilename, sourceFilename, platformPath = path_1.default) {
        // Replace asset path with new path relative to the output CSS
        const relative = platformPath.relative(platformPath.dirname(outputFilename), 
        // Get the absolute path to the asset from the path relative to the JS file
        platformPath.resolve(platformPath.dirname(sourceFilename), url));
        if (platformPath.sep === posixSep) {
            return relative;
        }
        return relative.split(platformPath.sep).join(posixSep);
    }
}
exports.default = getProcessorCssText;
//# sourceMappingURL=linaria-process-css.js.map