"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const generator_1 = __importDefault(require("@babel/generator"));
const core_1 = require("@babel/core");
const logger_1 = require("@linaria/logger");
const build_options_1 = __importDefault(require("./build-options"));
const shake = require('@linaria/shaker/lib/shaker').default;
function prepareForShake(filename, options, code) {
    const transformOptions = build_options_1.default(filename, options);
    transformOptions.ast = true;
    transformOptions.presets.unshift([
        require.resolve('@babel/preset-env'),
        {
            targets: 'ie 11',
            // we need this plugin so we list it explicitly, explanation in `packages/extractor/src/index`
            include: ['@babel/plugin-transform-template-literals']
        }
    ]);
    transformOptions.presets.unshift([require.resolve('./preeval'), options]);
    transformOptions.plugins.unshift(require.resolve('babel-plugin-transform-react-remove-prop-types'));
    transformOptions.plugins.unshift([
        require.resolve('@babel/plugin-transform-runtime'),
        { useESModules: false }
    ]);
    logger_1.debug('evaluator:shaker:transform', `Transform ${filename} with options ${JSON.stringify(transformOptions, null, 2)}`);
    const transformed = core_1.transformSync(code, transformOptions);
    if (transformed === null || !transformed.ast) {
        throw new Error(`${filename} cannot be transformed`);
    }
    return transformed.ast.program;
}
const shaker = (filename, options, text, only = null) => {
    const [shaken, imports] = shake(prepareForShake(filename, options, text), only);
    logger_1.debug('evaluator:shaker:generate', `Generate shaken source code ${filename}`);
    const { code: shakenCode } = generator_1.default(shaken);
    return [shakenCode, imports];
};
exports.default = shaker;
//# sourceMappingURL=index.js.map