"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const babel_preset_1 = require("@linaria/babel-preset");
// Verify if the binding is imported from the specified source
function hasImport(state, t, scope, filename, identifier) {
    const sources = identifier === 'css'
        ? state.options.importMap.cssLibs
        : state.options.importMap.reactLibs;
    const linariaLibs = [
        ...state.options.importMap.cssLibs,
        ...state.options.importMap.reactLibs
    ];
    const binding = scope.getAllBindings()[identifier];
    if (!binding) {
        return false;
    }
    const p = binding.path;
    const resolveFromFile = (id) => {
        try {
            return babel_preset_1.Module._resolveFilename(id, {
                id: filename,
                filename,
                paths: babel_preset_1.Module._nodeModulePaths(path_1.dirname(filename))
            });
        }
        catch (e) {
            return null;
        }
    };
    const isImportingModule = (value) => sources.some((source) => 
    // If the value is an exact match, assume it imports the module
    value === source ||
        // Otherwise try to resolve both and check if they are the same file
        resolveFromFile(value) ===
            (linariaLibs.indexOf(source) !== -1
                ? require.resolve(source)
                : resolveFromFile(source)));
    if (t.isImportSpecifier(p) && t.isImportDeclaration(p.parentPath)) {
        return isImportingModule(p.parentPath.node.source.value);
    }
    if (t.isVariableDeclarator(p)) {
        if (t.isCallExpression(p.node.init) &&
            t.isIdentifier(p.node.init.callee) &&
            p.node.init.callee.name === 'require' &&
            p.node.init.arguments.length === 1) {
            const node = p.node.init.arguments[0];
            if (t.isStringLiteral(node)) {
                return isImportingModule(node.value);
            }
            if (t.isTemplateLiteral(node) && node.quasis.length === 1) {
                return isImportingModule(node.quasis[0].value.cooked);
            }
        }
    }
    return false;
}
exports.default = hasImport;
//# sourceMappingURL=hasImport.js.map